@@ -13,7 +13,7 @@
 #define SYSTEM_FREQ 44100
 
 Generator::Generator(float secs, int freq)
-    :QIODevice( )
+    :QIODevice( ), isGenerating(false)
 {
     finished = false;
     buffer = new char[int(secs * SYSTEM_FREQ * 4) + 3000];
@@ -25,7 +25,7 @@ Generator::Generator(float secs, int freq)
 }
 
 Generator::Generator(Generator *copyFrom)
-    : QIODevice()
+    : QIODevice(), isGenerating(false)
 {
     buffer = new char[copyFrom->len];
     memcpy(buffer, copyFrom->buffer, copyFrom->len);
@@ -48,6 +48,7 @@ void Generator::clearBuffer() {
     t = buffer;
     len = bytes_left = 4;
     pos = 0;
+    isGenerating = false;
 }
 
 void Generator::appendDataFrom(const Generator *copyFrom) {
@@ -64,11 +65,13 @@ void Generator::appendDataFrom(const Generator *copyFrom) {
 void Generator::start()
 {
     open(QIODevice::ReadOnly);
+    isGenerating = true;
 }
 
 void Generator::stop()
 {
     close();
+    isGenerating = false;
 }
 
 int Generator::putShort(char *t, unsigned int value)
@@ -82,25 +85,24 @@ int Generator::fillData(char *start, int frequency, float seconds)
 {
     int i, len=0;
     int value;
-    for(i=0; i<int(seconds*SYSTEM_FREQ); i++) {
+    int ramp_samples = (int)(8e-3 * SYSTEM_FREQ); // ramp for 8ms
+    int total_samples = int(seconds*SYSTEM_FREQ);
+    
+    for(i=0; i<total_samples; i++) {
         if (frequency == 0.0)
             value = 0;
         else
             value=(int)(32767.0*sin(2.0*M_PI*((double)(i))*(double)(frequency)/SYSTEM_FREQ));
+	if (i < ramp_samples || total_samples - i <= ramp_samples) {
+            int filter_sample = i < ramp_samples ? i : (total_samples - i - 1);
+            double filter = (1 - cos(M_PI * (double)filter_sample / (double)ramp_samples)) / 2;
+
+            value *= filter;
+        }
         putShort(start, value);
         start += 2;
         len+=2;
     }
-    if (frequency > 100) {
-        // ramp down or up to 0 for a better drop off in sound
-        while(value > 8 || value < -8) {
-            i++;
-            value=(int)(32767.0*sin(2.0*M_PI*((double)(i))*(double)(frequency)/SYSTEM_FREQ));
-            putShort(start, value);
-            len += 2;
-            start += 2;
-        }
-    }
     bytes_left = len;
     pos = 0;
     return len;
@@ -120,15 +122,18 @@ qint64 Generator::readData(char *data, qint64 maxlen)
 
     //qDebug() << "left: " << bytes_left << " / wanted: " << len;
 
-    if (bytes_left == 0) {
+    if (bytes_left == -1 && isGenerating) {
+        isGenerating = false;
         emit generatorDone();
     }
 
+#undef FILL_WITH_SPACE_ONCE
+#undef ALWAYS_FILL_WITH_SPACE
+
 #ifdef Q_OS_LINUX
-    // On linux (with Qt 4.7 and 4.7.1) there is a nasty second-long pause/freeze after the audio finishes playing
+    // On linux (with Qt 4.7 and 4.7.1) there is a nasty second-long pause/freeze after the audio finishes playing, so
+    // we continue to emit empty sound endlessly to get around the gui/qt lockup.
 #define ALWAYS_FILL_WITH_SPACE 1
-#else
-#define FILL_WITH_SPACE_ONCE 0
 #endif
 
 #ifdef ALWAYS_FILL_WITH_SPACE
@@ -138,33 +143,41 @@ qint64 Generator::readData(char *data, qint64 maxlen)
         bytes_left = -1;
         return maxlen;
     }
-#elif FILL_WITH_SPACE_ONCE
+#elif defined(FILL_WITH_SPACE_ONCE)
     /* fill with a blank space just once after the starting */
-    if (bytes_left < 0) {
+    if (bytes_left == 0) {
         memset(data, 0, maxlen);
         bytes_left = -1;
         return maxlen;
     }
+    if (bytes_left == -1)
+        return -1;
 #else
     /* this is how it *should* be done, if the Qt output buffers didn't truncate things */
     if (bytes_left == 0)
         bytes_left = -1;
-    if (bytes_left <= 0)
+    if (bytes_left <= 0) {
+        if (isGenerating) {
+            isGenerating = false;
+            emit generatorDone();
+        }
         return -1;
+    }
 #endif
 
     if (len < bytes_left) {
         // Normal
-        memcpy(data,t+pos,len);
+        memcpy(data, t+pos, len);
         pos += len;
         bytes_left -= len;
         return len;
     } else {
         // Whats left
-        memcpy(data,t+pos,bytes_left);
+        memcpy(data, t+pos, bytes_left);
+        int to_return = bytes_left;
         bytes_left = 0;
         pos=0;
-        return bytes_left;
+        return to_return;
     }
 }
 
