@@ -21,16 +21,13 @@ public:
     explicit MainWindow(QWidget *parent = 0);
     ~MainWindow();
 
-    void createAudioOutput();
+    QAudioOutput *createAudioOutput();
     void keyPressEvent(QKeyEvent *);
     bool event(QEvent *event);
 
 public slots:
     void startIt();
 
-protected:
-    void closeEvent(QCloseEvent *event);
-
 private:
     Ui::MainWindow *ui;
     Ui::Prefs      *prefs;
