@@ -1,6 +1,8 @@
 #ifndef LETTERTRAININGMODE_H
 #define LETTERTRAININGMODE_H
 
+#include <QtGui/QSpinBox>
+
 #include "TrainingMode.h"
 
 class LetterTrainingMode : public TrainingMode
@@ -13,21 +15,32 @@ public:
 
     virtual void switchToMode();
     virtual void handleKeyPress(QChar letterPressed);
-    virtual QTime startNextTrainingKey();
     virtual bool elapsedTimeWasTooLong(int msElapsed, MorseStat *stat);
 
     virtual QString name();
     virtual QString helpText();
+    virtual QString icon();
 
     virtual void loadSettings(QSettings &settings);
     virtual void saveSettings(QSettings &settings);
 
+    virtual QBoxLayout * getPrefsLayout();
+    virtual void acceptPrefs();
+
 public slots:
     virtual void updateGraphs();
+    virtual void startNextTrainingKey();
+    virtual void play();
 
 private:
-    static const int m_percentGoal = 90;
-    static const int m_minimumTries = 10;
+    int m_percentGoal;
+    int m_minimumTries;
+    float m_letterDelay;
+
+    QSpinBox *m_percentGoalBox;
+    QSpinBox *m_minimumTriesBox;
+    QDoubleSpinBox *m_letterDelayBox;
+
 };
 
 #endif // LETTERTRAININGMODE_H
