@@ -8,13 +8,14 @@
 #include <QtGui/QSpinBox>
 #include <QtGui/QLabel>
 #include <QtGui/QScrollArea>
+#include <QtGui/QHBoxLayout>
 #include <qdebug.h>
 
 MorseMode::MorseMode(Morse *morse, Ui::MainWindow *ui)
     : m_morse(morse), m_ui(ui),
-      m_badCount(0), m_goodCount(0), m_countWeight(100),
+      m_badCount(0), m_goodCount(0),
       m_playIcon(":/icons/play.png"), m_pauseIcon(":/icons/pause.png"), m_runningMode(PAUSED),
-      m_mapper(new QSignalMapper()), m_buttons(0)
+      m_mapper(new QSignalMapper()), m_buttons(0), m_modeSpecificMenu(0)
 {
     setRunningMode(PAUSED);
     m_WPM = m_morse->currentWPMGoal();
@@ -43,10 +44,6 @@ void MorseMode::play()
 {
 }
 
-void MorseMode::pause()
-{
-}
-
 void MorseMode::stop()
 {
 }
@@ -143,7 +140,7 @@ void MorseMode::switchToYou()
 void MorseMode::createMenuStructures()
 {
 #ifdef SMALL_DEVICE
-    m_helpMenu = m_optionsMenu = m_cuteCWMenu = m_morse->menuBar();
+    m_helpMenu = m_optionsMenu = m_cuteCWMenu = m_modeSpecificMenu = m_morse->menuBar();
     // m_morse->createModesMenu(m_morse->menuBar()->addMenu(tr("Change Mode")));
 #else
     QMenuBar *topBar = m_morse->menuBar();
@@ -155,6 +152,19 @@ void MorseMode::createMenuStructures()
 #endif
 }
 
+#ifdef SMALL_DEVICE
+QMenuBar *MorseMode::menu() {
+    return m_morse->menuBar();
+}
+#else
+QMenu *MorseMode::menu() {
+    if (!m_modeSpecificMenu)
+        m_modeSpecificMenu = m_morse->menuBar()->addMenu(name());
+
+    return m_modeSpecificMenu;
+}
+#endif
+
 void MorseMode::modeMenus()
 {
 }
@@ -184,6 +194,13 @@ void MorseMode::switchFromYou()
     m_morse->menuBar()->clear();
     clearModeLayout();
 
+    // clear the mode specific menu
+    if (m_modeSpecificMenu) {
+        m_modeSpecificMenu->clear();
+        delete m_modeSpecificMenu;
+        m_modeSpecificMenu = 0;
+    }
+
     // stop the audio
     m_morse->pauseAudio();
     setRunningMode(PAUSED);
@@ -340,3 +357,67 @@ void MorseMode::saveSettings(QSettings &settings) {
 void MorseMode::loadSettings(QSettings &settings) {
     Q_UNUSED(settings);
 }
+
+void MorseMode::setupSequenceLayout(QVBoxLayout *parentLayout, QWidget *theMainThing)
+{
+    if (parentLayout == 0)
+        parentLayout = m_ui->verticalLayout;
+    if (theMainThing == 0)
+        theMainThing = m_ui->centralWidget;
+
+    m_sequenceLayout = new QHBoxLayout(theMainThing);
+    parentLayout->addLayout(m_sequenceLayout);
+    m_sequenceLayout->addWidget(new QLabel(tr("Sequence:")));
+    m_sequenceLayout->addWidget(m_sequenceLabel = new QLabel(tr("")));
+}
+
+void MorseMode::setupLastWPMLayout(QVBoxLayout *parentLayout, QWidget *theMainThing)
+{
+    if (parentLayout == 0)
+        parentLayout = m_ui->verticalLayout;
+    if (theMainThing == 0)
+        theMainThing = m_ui->centralWidget;
+
+    m_lastWPMLayout = new QHBoxLayout(theMainThing);
+    parentLayout->addLayout(m_lastWPMLayout);
+    m_lastWPMLayout->addWidget(new QLabel(tr("Last WPM:")));
+    m_lastWPMLayout->addWidget(m_lastwpmLabel = new QLabel(tr("")));
+}
+
+void MorseMode::setupAveWPMLayout(QVBoxLayout *parentLayout, QWidget *theMainThing)
+{
+    if (parentLayout == 0)
+        parentLayout = m_ui->verticalLayout;
+    if (theMainThing == 0)
+        theMainThing = m_ui->centralWidget;
+
+    m_aveWPMLayout = new QHBoxLayout(theMainThing);
+    parentLayout->addLayout(m_aveWPMLayout);
+    m_aveWPMLayout->addWidget(new QLabel(tr("Average WPM:")));
+    m_aveWPMLayout->addWidget(m_avewpmLabel = new QLabel(tr("")));
+}
+
+void MorseMode::setupSequenceLayouts(QVBoxLayout *parentLayout, QWidget *theMainThing)
+{
+    setupSequenceLayout(parentLayout, theMainThing);
+    setupLastWPMLayout(parentLayout, theMainThing);
+    setupAveWPMLayout(parentLayout, theMainThing);
+}
+
+QBoxLayout *MorseMode::getPrefsLayout()
+{
+    return 0;
+}
+
+void MorseMode::acceptPrefs()
+{
+}
+
+void MorseMode::rejectPrefs()
+{
+}
+
+QString MorseMode::icon()
+{
+    return QString();
+}
