@@ -28,3 +28,8 @@ QString PlayMode::name()
 {
     return tr("Type Morse Code");
 }
+
+QString PlayMode::icon()
+{
+    return ":/icons/64x64/type.png";
+}
