@@ -2,12 +2,14 @@
 
 #include <QtGui/QFileDialog>
 #include <QtGui/QMessageBox>
+#include <QtGui/QFormLayout>
+#include <QtCore/QTimer>
 
 #include "ReadMode.h"
 #include "Morse.h"
 
 ReadMode::ReadMode(Morse *parent, Ui::MainWindow *ui)
-    : MorseMode(parent, ui)
+    : MorseMode(parent, ui), m_textEdit(0), m_readWordCount(1)
 {
 }
 
@@ -52,36 +54,52 @@ ReadMode::openFile() {
 }
 
 void ReadMode::play() {
-    QTextCursor readSpot = m_textEdit->textCursor();
-    readSpot.select(QTextCursor::Document);
-
-    m_morse->add(readSpot.selectedText());
-    m_morse->playSequence();
+    m_readSpot = m_textEdit->textCursor();
+    if (m_readSpot.atEnd())
+        m_readSpot.movePosition(QTextCursor::Start);
+    readWordUnderCursor();
+    m_currentWordCount = 1;
     return;
-    //qDebug() << "starting to read";
-    //m_readSpot = m_ui->wordbox->cursorForPosition(QPoint(0,0));
-    //readNextLetter();
 }
 
-void ReadMode::readNextLetter() {
-    if (m_readSpot.atEnd()) {
-        qDebug() << "reached the end; stopping playing";
-        return;
+void ReadMode::stop() {
+    if (m_textEdit && !m_readSpot.isNull()) {
+        m_textEdit->setTextCursor(m_readSpot);
+        m_readSpot.select(QTextCursor::WordUnderCursor);
+        m_readSpot.insertHtml(m_currentText);
     }
-    // move the anchor to where the cursor is
-    m_readSpot.movePosition(QTextCursor::NoMove, QTextCursor::MoveAnchor, 0);
-    // move the selection pointer to the right one, highlighting the current
-    // selection letter we're going to play
-    m_readSpot.movePosition(QTextCursor::Right, QTextCursor::KeepAnchor, 1);
-    // play the selection
-    qDebug() << "playing selected text: " << m_readSpot.selectedText();
-    m_morse->addAndPlayIt(m_readSpot.selectedText()[0]);
+}
+
+void ReadMode::readWordUnderCursor() {
+    m_readSpot.select(QTextCursor::WordUnderCursor);
+    m_morse->add(m_readSpot.selectedText());
+    m_morse->playSequence();
+    QTextCharFormat format = m_readSpot.blockCharFormat();
+    format.setFontUnderline(true);
+    //m_readSpot.setBlockCharFormat(format);
+    m_currentText = m_readSpot.selectedText();
+    m_readSpot.insertHtml("<u>" + m_currentText + "</u>");
 }
 
 void ReadMode::audioStopped()
 {
-    qDebug() << "audio stopped method";
-    setRunningMode(PAUSED);
+    // decide if we're going to re-read the current word or not.
+    if (m_currentWordCount >= m_readWordCount) {
+        m_currentWordCount = 1;
+        m_readSpot.select(QTextCursor::WordUnderCursor);
+        m_readSpot.insertHtml(m_currentText);
+        m_currentText = "";
+        m_readSpot.movePosition(QTextCursor::NextWord);
+    } else {
+        m_currentWordCount++;
+    }
+
+    if (!m_readSpot.atEnd()) {
+        QTimer::singleShot(int(m_morse->spaceSecs() * 1000.0), this, SLOT(readWordUnderCursor()));
+    } else {
+        m_textEdit->setTextCursor(m_readSpot);
+        setRunningMode(PAUSED);
+    }
 }
 
 QString ReadMode::helpText()
@@ -99,3 +117,36 @@ QString ReadMode::name()
     return tr("Read to me");
 }
 
+QString ReadMode::icon()
+{
+    return ":/icons/64x64/read-to-me.png";
+}
+
+
+void ReadMode::loadSettings(QSettings &settings)
+{
+    m_readWordCount = settings.value("readtome/readWordCount", 1).toInt();
+}
+
+void ReadMode::saveSettings(QSettings &settings)
+{
+    settings.setValue("readtome/readWordCount", m_readWordCount);
+}
+
+QBoxLayout *ReadMode::getPrefsLayout()
+{
+    QHBoxLayout *hbox = new QHBoxLayout();
+    QFormLayout *form = new QFormLayout();
+    hbox->addLayout(form);
+
+    form->addRow(tr("Read each word this many times:"), m_readWordCountBox = new QSpinBox());
+    m_readWordCountBox->setRange(1,10);
+    m_readWordCountBox->setValue(m_readWordCount);
+
+    return hbox;
+}
+
+void ReadMode::acceptPrefs()
+{
+    m_readWordCount = m_readWordCountBox->value();
+}
