@@ -1,13 +1,19 @@
 #include <qdebug.h>
+#include <QtGui/QHBoxLayout>
+#include <QtGui/QLabel>
+#include <QtGui/QFormLayout>
 
 #include "SpeedTrainingMode.h"
 
 SpeedTrainingMode::SpeedTrainingMode(Morse *parent, Ui::MainWindow *ui)
-    : TrainingMode(parent, ui)
+    : TrainingMode(parent, ui), m_countWeight(50), m_speedMultiplier(1.5)
 {
 }
 
 void SpeedTrainingMode::switchToMode() {
+
+    setupTrainingWidgets();
+
     m_ui->letter->show();
     m_ui->changeSequence->show();
     //m_ui->forModes->show();
@@ -17,11 +23,12 @@ void SpeedTrainingMode::switchToMode() {
 
     setupSequences();
     setupWidgets(m_trainingSequence, true, "S\np\ne\ne\nd\n \nR\na\nt\ni\nn\ng");
-    setupTrainingWidgets();
     updateGraphs();
 }
 
 void SpeedTrainingMode::play() {
+    m_lastKeys.clear();
+    m_lastTimes.clear();
     startNextSpeedKey();
 }
 
@@ -44,10 +51,14 @@ void SpeedTrainingMode::startTimerToNextKey(int plusMSecs) {
     if (runningMode() != RUNNING)
         return;
 
+    if (plusMSecs <= 0) {
+        qDebug() << "stop here!";
+        plusMSecs = QTime::currentTime().msecsTo(m_morse->sequenceTime());
+    }
     if (m_lastTimes.count() > 3) {
         // don't let them get *too* far behind
         m_badCount++;
-        QTimer::singleShot(1000, this, SLOT(startNextSpeedKey()));  // Try again in a second
+        QTimer::singleShot(1000, this, SLOT(startTimerToNextKey()));  // Try again in a second
         return;
     }
     avetime = getStat(m_lastKey)->getAverageTime();
@@ -56,8 +67,9 @@ void SpeedTrainingMode::startTimerToNextKey(int plusMSecs) {
         avetime = 1000.0;
         qDebug() << "setting avetime to: " << avetime;
     }
-    delay  = plusMSecs + (float(m_badCount + m_countWeight)/float(m_goodCount + m_countWeight)) * avetime;
-    qDebug() << "delaying for: " << delay << " ms (good=" << m_goodCount << ", bad=" << m_badCount << ")";
+    delay  = plusMSecs + (float(m_badCount + m_countWeight)/float(m_goodCount + m_countWeight)) * avetime * m_speedMultiplier;
+    qDebug() << "delaying for: t=" << avetime << " + ex=" << plusMSecs << " = " << delay << " ms (good=" << m_goodCount << ", bad=" << m_badCount
+             << ", wt=" << m_countWeight << ")";
     QTimer::singleShot(delay, this, SLOT(startNextSpeedKey()));
 }
 
@@ -82,6 +94,11 @@ QString SpeedTrainingMode::name()
     return tr("Speed Training");
 }
 
+QString SpeedTrainingMode::icon()
+{
+    return ":/icons/64x64/speed.png";
+}
+
 QTime SpeedTrainingMode::startNextTrainingKey() {
     if (runningMode() != RUNNING)
         return QTime();
@@ -120,14 +137,14 @@ QTime SpeedTrainingMode::startNextTrainingKey() {
             m_lastKey = *letter;
             m_lastKeys.append(*letter);
             setSequence(m_trainingSequence, letterCount);
-            m_ui->avewpm->setText("All WPM: " + QString().setNum(msToPauseWPM(totalTime/letterCount)) + ", " +
+            m_avewpmLabel->setText("All WPM: " + QString().setNum(msToPauseWPM(totalTime/letterCount)) + ", " +
                                   *letter + ": NEW");
             if (m_morse->trainingMode() == Morse::SPEEDTRAIN)
                 setWPMLabel(msToPauseWPMF((float(m_badCount + m_countWeight)/float(m_goodCount + m_countWeight)) *
                                           totalTime/float(letterCount)));
             else
                 setWPMLabel(msToPauseWPMF(totalTime/float(letterCount)));
-            m_lastTimes.push_back(m_morse->playIt(*letter));
+            m_lastTimes.push_back(m_morse->playIt(*letter, false));
             updateGraphs();
             return m_lastTimes.last();
         }
@@ -135,7 +152,7 @@ QTime SpeedTrainingMode::startNextTrainingKey() {
         //qDebug() << "  adding " << *letter << " / " << thisTime << " / " << msToPauseWPM(thisTime);
         letters.append(QPair<QChar, float>(*letter, thisTime));
 
-        if(msToPauseWPM(thisTime) <= m_morse->currentWPMAccept()) {
+        if(msToPauseWPM(thisTime) < m_morse->currentWPMAccept()) {
             qDebug() << " too slow: " << *letter << " / " << thisTime << " / " << msToPauseWPM(thisTime);
             if (++badLetters >= m_maxBadLetters) {
                 // we're not fast enough; break here
@@ -167,14 +184,13 @@ QTime SpeedTrainingMode::startNextTrainingKey() {
         totalWPM += pauseWPM;
     }
 
-    m_ui->avewpm->setText("All WPM: " + QString().setNum(msToPauseWPM(totalTime/letterCount)) + ", " +
+    m_avewpmLabel->setText("All WPM: " + QString().setNum(msToPauseWPM(totalTime/letterCount)) + ", " +
                           currentLetterGoal + " WPM: " + QString().setNum(msToPauseWPM(thisTime)));
     setWPMLabel(msToPauseWPMF((float(m_badCount + m_countWeight)/float(m_goodCount + m_countWeight)) * totalTime/float(letterCount)));
 
     // now pick a random time between 0 and the total of all the averages; averages with a slower speed are more likely
-    // XXX: probably could use a weighted average (subtract off min speed from all speeds)?
 
-    float randWPM, subTime = 0.0;
+    float randWPM;
 
     randWPM = totalWPM*float(qrand())/float(RAND_MAX);
 
@@ -189,7 +205,7 @@ QTime SpeedTrainingMode::startNextTrainingKey() {
             qDebug() << ">keying: " << (*search).first;
             m_lastKey = (*search).first;
             m_lastKeys.append((*search).first);
-            m_lastTimes.push_back(m_morse->playIt((*search).first));
+            m_lastTimes.push_back(m_morse->playIt((*search).first, false));
             updateGraphs();
             return m_lastTimes.last();
         }
@@ -233,10 +249,52 @@ void SpeedTrainingMode::updateGraphs()
 
 void SpeedTrainingMode::loadSettings(QSettings &settings)
 {
+    m_countWeight = settings.value("speedtraining/countWeight", 50).toInt();
+    m_speedMultiplier = settings.value("speedtraining/speedWeight", 1.5).toFloat();
     loadStats(settings);
 }
 
 void SpeedTrainingMode::saveSettings(QSettings &settings)
 {
+    settings.setValue("speedtraining/countWeight", m_countWeight);
+    settings.setValue("speedtraining/speedWeight", m_speedMultiplier);
     saveStats(settings);
 }
+
+QBoxLayout *SpeedTrainingMode::getPrefsLayout()
+{
+    QHBoxLayout *hbox = new QHBoxLayout();
+    QFormLayout *form = new QFormLayout();
+    hbox->addLayout(form);
+
+    form->addRow(tr("Speed Acceptance Rate"), m_acceptRateBox = new QSpinBox());
+    m_acceptRateBox->setRange(1,100);
+    m_acceptRateBox->setValue(m_morse->currentWPMAccept());
+
+    form->addRow(tr("Speed Increasing Aggressiveness"), m_countWeightBox = new QSpinBox());
+    m_countWeightBox->setRange(1,1000);
+    m_countWeightBox->setValue(m_countWeight);
+
+    QLabel *helpLabel = new QLabel(tr("(lower values mean speed increases more quickly)"));
+    QFont font = helpLabel->font();
+    font.setItalic(true);
+    helpLabel->setFont(font);
+    form->addRow("", helpLabel);
+
+    form->addRow(tr("Speed Multiplier"), m_speedMultiplierBox = new QDoubleSpinBox());
+    m_speedMultiplierBox->setRange(.1, 5.0);
+    m_speedMultiplierBox->setValue(m_speedMultiplier);
+
+    helpLabel = new QLabel(tr("(higher forces a slower starting speed)"));
+    helpLabel->setFont(font);
+    form->addRow("", helpLabel);
+
+    return hbox;
+}
+
+void SpeedTrainingMode::acceptPrefs()
+{
+    m_morse->setWPMAccept(m_acceptRateBox->value());
+    m_countWeight = m_countWeightBox->value();
+    m_speedMultiplier = m_speedMultiplierBox->value();
+}
