@@ -1,6 +1,8 @@
 #ifndef SPEEDTRAININGMODE_H
 #define SPEEDTRAININGMODE_H
 
+#include <QtGui/QSpinBox>
+
 #include "TrainingMode.h"
 #include "Morse.h"
 
@@ -18,6 +20,7 @@ public:
 
     virtual QString name();
     virtual QString helpText();
+    virtual QString icon();
 
     virtual QTime startNextTrainingKey();
     void startTimerToNextKey(QTime plusTime);
@@ -26,10 +29,20 @@ public:
     virtual void loadSettings(QSettings &settings);
     virtual void saveSettings(QSettings &settings);
 
+    virtual QBoxLayout * getPrefsLayout();
+    virtual void acceptPrefs();
 public slots:
     void startTimerToNextKey(int plusMSecs = 0);
     void startNextSpeedKey();
     virtual void updateGraphs();
+
+private:
+    int                             m_countWeight;
+    float                           m_speedMultiplier;
+
+    QSpinBox *m_acceptRateBox;
+    QSpinBox *m_countWeightBox;
+    QDoubleSpinBox *m_speedMultiplierBox;
 };
 
 #endif // SPEEDTRAININGMODE_H
