@@ -7,7 +7,7 @@
 #include <QtGui/QProgressBar>
 
 TrainingMode::TrainingMode(Morse *parent, Ui::MainWindow *ui)
-    : MorseMode(parent, ui), m_doEntireSequence(false), m_maxBadLetters(2), m_includeProgressBars(true)
+    : MorseMode(parent, ui), m_doEntireSequence(false), m_maxBadLetters(2), m_includeProgressBars(true), m_acceptForwardKeys(true)
 {
     setupSequences();
 }
@@ -26,6 +26,11 @@ void TrainingMode::setupSequences() {
     m_sequences.append("tuvwxyz");
     m_sequences.append("1234567890");
     m_sequences.append(".,/=?");
+    m_sequences.append("eish5");
+    m_sequences.append("tmo0");
+    m_sequences.append("auv4");
+    m_sequences.append("ndb6");
+    m_sequences.append("awj1");
     m_sequences.append("abcdefghijklmnopqrstuvwxyz1234567890.,/=?");
 
     // Koch sequences
@@ -79,6 +84,30 @@ void TrainingMode::setupSequences() {
     connect(action, SIGNAL(triggered()), m_sequenceSignalMapper, SLOT(map()));
     m_sequenceSignalMapper->setMapping(action, (int) UZ);
 
+    // Similar
+    subMenu = modeMenu->addMenu(tr("Similar"));
+
+    action = subMenu->addAction("EISH5");
+    connect(action, SIGNAL(triggered()), m_sequenceSignalMapper, SLOT(map()));
+    m_sequenceSignalMapper->setMapping(action, (int) SIM1);
+
+    action = subMenu->addAction("TMO0");
+    connect(action, SIGNAL(triggered()), m_sequenceSignalMapper, SLOT(map()));
+    m_sequenceSignalMapper->setMapping(action, (int) SIM2);
+
+    action = subMenu->addAction("AUV4");
+    connect(action, SIGNAL(triggered()), m_sequenceSignalMapper, SLOT(map()));
+    m_sequenceSignalMapper->setMapping(action, (int) SIM3);
+
+    action = subMenu->addAction("NDB6");
+    connect(action, SIGNAL(triggered()), m_sequenceSignalMapper, SLOT(map()));
+    m_sequenceSignalMapper->setMapping(action, (int) SIM4);
+
+    action = subMenu->addAction("AWJ1");
+    connect(action, SIGNAL(triggered()), m_sequenceSignalMapper, SLOT(map()));
+    m_sequenceSignalMapper->setMapping(action, (int) SIM5);
+
+
     // numbers and symbols
 
     action = modeMenu->addAction("Numbers");
@@ -89,10 +118,12 @@ void TrainingMode::setupSequences() {
     connect(action, SIGNAL(triggered()), m_sequenceSignalMapper, SLOT(map()));
     m_sequenceSignalMapper->setMapping(action, (int) SYMBOLS);
 
-    action = modeMenu->addAction("Symbols");
+    action = modeMenu->addAction("Everything");
     connect(action, SIGNAL(triggered()), m_sequenceSignalMapper, SLOT(map()));
     m_sequenceSignalMapper->setMapping(action, (int) EVERYTHING);
 
+    // Custom
+
     action = modeMenu->addAction("Custom");
     connect(action, SIGNAL(triggered()), this, SLOT(chooseCustomeSequence()));
 
@@ -101,7 +132,20 @@ void TrainingMode::setupSequences() {
 }
 
 void TrainingMode::setupTrainingWidgets() {
-    connect(m_optionsMenu->addAction(tr("Clear Training")), SIGNAL(triggered()), this, SLOT(clear()));
+    setupSequenceLayouts();
+
+    m_ui->letter->setText("");
+    m_ui->WPM->setText("");
+}
+
+void TrainingMode::modeMenus() {
+    connect(menu()->addAction(tr("Clear Training")), SIGNAL(triggered()), this, SLOT(clear()));
+
+    // Create the preference items in the quick menu
+    m_doEntireSequenceButton = menu()->addAction(tr("Use &Entire Sequence"));
+    m_doEntireSequenceButton->setCheckable(true);
+    m_doEntireSequenceButton->setChecked(false);
+    connect(m_doEntireSequenceButton, SIGNAL(toggled(bool)), this, SLOT(setDoEntireSequence(bool)));
 }
 
 void TrainingMode::chooseCustomeSequence() {
@@ -132,17 +176,13 @@ void TrainingMode::clear()  {
     m_goodCount = 0;
     m_badCount = 0;
 
-    m_ui->avewpm->setText("All WPM: [None], K WPM: NEW");
+    m_avewpmLabel->setText("All WPM: [None], K WPM: NEW");
 
     if (runningMode() == RUNNING)
         playButton(); // pretend a pause was pressed too
     updateGraphs();
 }
 
-void TrainingMode::play() {
-    startNextTrainingKey();
-}
-
 void TrainingMode::audioStopped() {
     qDebug() << "audio stopped";
     // now done based on predicted time
@@ -176,10 +216,36 @@ void TrainingMode::handleKeyPress(QChar letterPressed) {
     qDebug() << "Training response: elapsed " << msElapsed << "ms (" << msToPauseWPM(msElapsed) << " WPM)";
     MorseStat *pressedStat = getStat(letterPressed);
 
-    // if the user took a *really* long time, ignore the key press and assume they got distracted from training
-    if (elapsedTimeWasTooLong(msElapsed, pressedStat)) {
-        qDebug() << "ignoring key press; too long and probably an interruption";
-        return;
+    if (m_acceptForwardKeys && letterPressed != lastKey && m_lastKeys.count() > 0) {
+        int whichLetter = 0;
+        bool foundOne = false;
+
+        foreach(QChar thekey, m_lastKeys) {
+            if (letterPressed == thekey) {
+                // We found the key they pressed that was correct.
+                foundOne = true;
+                break;
+            }
+            whichLetter++;
+        }
+
+        if (foundOne) {
+            // drop (and penalize) all the earlier ones
+            for(int i = 0; i < whichLetter; i++) {
+                QChar theLetter = m_lastKeys.takeFirst();
+                lastTime = m_lastTimes.takeFirst();
+                getStat(theLetter)->addStat(3.0 * getStat(theLetter)->getAverageTime(), false);
+                m_badCount++;
+            }
+
+            // penalize the key previously pulled off
+            getStat(lastKey)->addStat(3.0 * getStat(lastKey)->getAverageTime(), false);
+            m_badCount++;
+
+            // pull the new matching key off
+            lastKey = m_lastKeys.takeFirst();
+            lastTime = m_lastTimes.takeFirst();
+        }
     }
 
     // set the last WPM record on the display
@@ -188,7 +254,13 @@ void TrainingMode::handleKeyPress(QChar letterPressed) {
         WPM = WPM + "-";
     }
     qDebug() << "WPM text: " << WPM;
-    m_ui->lastwpm->setText(WPM);
+    m_lastwpmLabel->setText(WPM);
+
+    // if the user took a *really* long time, ignore the key press and assume they got distracted from training
+    if (elapsedTimeWasTooLong(msElapsed, pressedStat)) {
+        qDebug() << "ignoring key press; too long and probably an interruption";
+        return;
+    }
 
     // if the keyed incorrectly, penalize them 3 times their average else add in the results
     if (letterPressed == lastKey) {
@@ -219,9 +291,7 @@ void TrainingMode::switchSequence(int sequence) {
 void TrainingMode::switchSequence(const QString &sequence) {
     m_trainingSequence = sequence;
     setSequence(m_trainingSequence, 1);
-    clear();
     setupModeWidgets(m_trainingSequence);
-    startNextTrainingKey();
 }
 
 void TrainingMode::setupModeWidgets(const QString &sequence, QString barLabel) {
@@ -232,16 +302,18 @@ void TrainingMode::setupModeWidgets(const QString &sequence, QString barLabel) {
     setupKeyWidgets(sequence, vbox);
     setSequence(sequence, 1);
 #ifndef SMALL_DEVICE
-    if (m_includeProgressBars)
+    if (m_includeProgressBars) {
         vbox->addLayout(setupGraphs(barLabel));
+        updateGraphs();
+    }
 #endif
 }
 
 void TrainingMode::setSequence(const QString &sequence, int currentlyAt) {
-    if (m_morse->m_sequenceLabel) {
+    if (m_sequenceLabel) {
         QString left = sequence.left(currentlyAt);
         QString right = sequence.right(sequence.length() - currentlyAt);
-        m_morse->m_sequenceLabel->setText("<font color=\"red\">" + left.toUpper() + "</font>" + right.toUpper());
+        m_sequenceLabel->setText("<font color=\"red\">" + left.toUpper() + "</font>" + right.toUpper());
 
         QChar theLetter = sequence[currentlyAt-1].toLower();
         QString newLetter = "<font color=\"red\">" + QString(theLetter.toUpper()) + "&nbsp;&nbsp;&nbsp;&nbsp;";
@@ -268,20 +340,15 @@ void TrainingMode::setupWidgets(const QString &sequence, bool includeProgressBar
 {
     m_includeProgressBars = includeProgressBars;
 
-    // Create the preference items in the quick menu
-    m_doEntireSequenceButton = m_optionsMenu->addAction(tr("Use &Entire Sequence"));
-    m_doEntireSequenceButton->setCheckable(true);
-    m_doEntireSequenceButton->setChecked(false);
-    connect(m_doEntireSequenceButton, SIGNAL(toggled(bool)), this, SLOT(setDoEntireSequence(bool)));
-
     setupModeWidgets(sequence, barLabel);
 }
 
-QGridLayout *TrainingMode::setupGraphs(QString barLabel)
+QBoxLayout *TrainingMode::setupGraphs(QString barLabel)
 {
     int column = 1;
     m_progressBars.clear();
     m_progressLabels.clear();
+    QHBoxLayout *hbox = new QHBoxLayout();
     QGridLayout *gridLayout = new QGridLayout();
 
     QLabel *leftLabel = new QLabel(barLabel);
@@ -304,13 +371,9 @@ QGridLayout *TrainingMode::setupGraphs(QString barLabel)
         m_progressBars[theLetter] = bar;
         m_progressLabels[theLetter] = label;
     }
-    return gridLayout;
-}
-
-
-QTime TrainingMode::startNextTrainingKey()
-{
-    exit(1); // should not ever be called
+    hbox->addLayout(gridLayout);
+    hbox->addStretch(1);
+    return hbox;
 }
 
 void TrainingMode::updateGraphs()
