@@ -19,9 +19,11 @@ void GroupGame::switchToMode() {
     GroupingMode::switchToMode();
     m_ui->helpBar->setText(tr("<font color=\"green\">Type the groups of characters you hear ASAP.</font>"));
     m_ui->changeSequence->hide();
+}
 
+void GroupGame::modeMenus() {
     // Create the high scores button
-    QAction *action = m_morse->menuBar()->addAction("HighScores");
+    QAction *action = menu()->addAction("HighScores");
     connect(action, SIGNAL(triggered()), &m_scores, SLOT(showScores()));
 }
 
@@ -64,7 +66,6 @@ void GroupGame::limitLength(int newValue) {
 
 void GroupGame::gameOver()
 {
-  pause();
   m_scores.addScore("", score());
 }
 
@@ -80,7 +81,7 @@ void GroupGame::groupGuessed(int right, int total)
 
 void GroupGame::setSequenceText()
 {
-  m_morse->m_sequenceLabel->setText(tr("sequence length = %1, WPM=%2, score = %3, turn=%4/%5").arg(m_groupLength).arg(m_WPM).arg(score()).arg(turn()).arg(maxTurns()));
+  m_sequenceLabel->setText(tr("sequence length = %1, WPM=%2, score = %3, turn=%4/%5").arg(m_groupLength).arg(m_WPM).arg(score()).arg(turn()).arg(maxTurns()));
 }
 
 QString GroupGame::helpText()
@@ -95,3 +96,8 @@ QString GroupGame::name()
 {
     return tr("Grouping game");
 }
+
+QString GroupGame::icon()
+{
+    return ":/icons/64x64/game.png";
+}
