@@ -1,6 +1,8 @@
 #ifndef GROUPGAMEMODE_H
 #define GROUPGAMEMODE_H
 
+#include <QtGui/QSpinBox>
+
 #include "modes/GroupingMode.h"
 #include "modes/games/MCountGameMode.h"
 #include "modes/games/HighScores.h"
@@ -13,9 +15,11 @@ public:
     explicit GroupGame(Morse *parent, Ui::MainWindow *ui);
 
     virtual void switchToMode();
+    virtual void modeMenus();
 
     virtual QString name();
     virtual QString helpText();
+    virtual QString icon();
 
 signals:
 
