@@ -4,6 +4,9 @@
 #include "modes/QModeStart.h"
 
 #include <QtGui/QMenuBar>
+#include <QtGui/QFormLayout>
+#include <QtGui/QVBoxLayout>
+#include <QtGui/QSpinBox>
 
 WordGame::WordGame(Morse *parent, Ui::MainWindow *ui) :
   WordTrainingMode(parent, ui), MCountGameMode(), m_scores("Word Accuracy Game")
@@ -16,12 +19,15 @@ void WordGame::switchToMode() {
     WordTrainingMode::switchToMode();
     m_ui->helpBar->setText(tr("<font color=\"green\">Type the groups of characters you hear ASAP.</font>"));
     m_ui->changeSequence->hide();
+}
 
+void WordGame::modeMenus() {
     // Create the high scores button
-    QAction *action = m_morse->menuBar()->addAction("HighScores");
+    QAction *action = menu()->addAction("HighScores");
     connect(action, SIGNAL(triggered()), &m_scores, SLOT(showScores()));
 }
 
+
 void WordGame::play()
 {
     startGame();
@@ -63,7 +69,7 @@ void WordGame::groupGuessed(int right, int total)
 
 void WordGame::setSequenceText()
 {
-    m_morse->m_sequenceLabel->setText(tr("WPM=%1, score = %2, turn=%3/%4").arg(m_WPM).arg(score()).arg(turn()).arg(maxTurns()));
+    m_sequenceLabel->setText(tr("WPM=%1, score = %2, turn=%3/%4").arg(m_WPM).arg(score()).arg(turn()).arg(maxTurns()));
     qDebug() << tr("WPM=%1, score = %2, turn=%3/%4").arg(m_WPM).arg(score()).arg(turn()).arg(maxTurns());
 }
 
@@ -77,3 +83,8 @@ QString WordGame::name()
 {
     return tr("Word Game");
 }
+
+QString WordGame::icon()
+{
+    return ":/icons/64x64/game.png";
+}
