@@ -13,9 +13,11 @@ public:
     explicit WordGame(Morse *parent, Ui::MainWindow *ui);
 
     virtual void switchToMode();
+    virtual void modeMenus();
 
     virtual QString name();
     virtual QString helpText();
+    virtual QString icon();
 
 signals:
 
